UNIT FileDisp;

INTERFACE

USES Crt,ScrLib,Dos,Utility,WindLib,IOLib;

PROCEDURE ScrollingFileList(    Template        : String;
                            VAR FileName        : String;
                            VAR Page,Loc        : Word;
                            VAR Result          : Byte;
                                Shadow          : Boolean);
IMPLEMENTATION

CONST
     MaxFileNames       =  500;
     FileNameLen        =   15;
     UlRow              =    2;
     ULCol              =    3;
     HowDeep            =    12;
     MaxDisplay         =  Pred(HowDeep) * (80 DIV FileNameLen);
     BoxColor    : Byte =  112;
     InnerColor  : Byte =   15;
     BarColor    : Byte =   112;
     InsertSortCriteria =   10;
TYPE
  String12  = String[12];
  FileList  = Array[1..MaxFileNames] OF String12;
  FileListPtr = ^FileList;

VAR
  NameArr            : FileListPtr;
  Holder1,Holder2    : String12;

(****  THE SORT STARTS HERE   ******)

{sorting requires a global array of string12 = NameArr^}

{StrComp returns TRUE if St1 < St2, otherwise it returns FALSE}

FUNCTION StrComp(VAR St1,St2 : String):Boolean;
         INLINE($8C/$DA        {MOV     DX,DS}
                /$5F           {POP     DI}
                /$07           {POP     ES}
                /$5E           {POP     SI}
                /$1F           {POP     DS}
                /$FC           {CLD      }
                /$AC           {LODSB    }
                /$26           {ES:      }
                /$8A/$25       {MOV AH,[DI]}
                /$47           {INC DI   }
                /$32/$ED       {XOR CH,CH}
                /$8A/$CC       {MOV CL,AH}
                /$3A/$C4       {CMP AL,AH}
                /$77/$02       {JA  0197 }
                /$8A/$C8       {MOV CL,AL}
                /$8A/$D8       {MOV BL,AL}
                /$32/$C0       {XOR AL,AL}
                /$F3           {REPZ     }
                /$A6           {CMPSB    }
                /$74/$04       {JZ  01A5 }
                /$79/$08       {JNS 01A1 }
                /$EB/$04       {JMP SHORT}
                /$2A/$DC       {SUB BL,AH}
                /$73/$02       {JAE  01B1}
                /$B0/$01       {MOV AL,01}
                /$8E/$DA);     {MOV DS,DX; 40 bytes total}

{this portion calls itself recursively - it is faster but much nastier
 to build a non-recursive quicksort. See Numerical Recipes, 1986, Press
 Et Al. Cambridge University Press for an example.  In this case, it might
 be useful to compare this sort with the one provided as an example
 program on your TP distribution disk (QSORT.PAS).  Notice that temporary
 strings are removed from the recursive procedure, this means that stack
 requirements don't vary with the type of element sorted.}

PROCEDURE RecursiveSort(Low,High : Word);
          VAR
             Ct1,Ct2       : Word;
          BEGIN

  {if the gap between the low and high markers is small enough, then
   it is more efficient to insertion sort}

                IF (High-Low < InsertSortCriteria) THEN BEGIN
                   FOR Ct1 := Succ(Low) TO High DO BEGIN
                       StringMove(NameArr^[Ct1],Holder1);
                       Ct2 := Pred(Ct1);
                       WHILE (StrComp(Holder1,NameArr^[Ct2])) AND (Ct2 > 0) DO
                             BEGIN
                             StringMove(NameArr^[Ct2],NameArr^[Succ(Ct2)]);
                             Dec(Ct2);
                       END;
                       Inc(Ct2);
                       IF Ct2 <> Ct1 THEN
                          StringMove(Holder1,NameArr^[Ct2]);
                   END;
                END
                ELSE BEGIN
                     Ct2 := Low;
                     Ct1 := High;

  {Picking an element from the middle of the segment eliminates the problems
   encountered quick-sorting ordered sets}

                     StringMove(NameArr^[(High + Low) DIV 2],Holder1);

 {Here is where most of the time is spent.  It turns out that avoiding a
  string compare by first checking the first letter is of advantage}

                     REPEAT
                           WHILE NameArr^[Ct2,1] < Holder1[1] DO
                                 Inc(Ct2);
                           WHILE StrComp(NameArr^[Ct2],Holder1) DO
                                 Inc(Ct2);
                           WHILE Holder1[1] < NameArr^[Ct1,1] DO
                                 Dec(Ct1);
                           WHILE StrComp(Holder1,NameArr^[Ct1]) DO
                                 Dec(Ct1);
                           IF Ct2 <= Ct1 THEN BEGIN
                              StringMove(NameArr^[Ct2],Holder2);
                              StringMove(NameArr^[Ct1],NameArr^[Ct2]);
                              StringMove(Holder2,NameArr^[Ct1]);
                              Inc(Ct2);
                              Dec(Ct1);
                           END;
                     UNTIL Ct1 < Ct2;
                     IF Low < Ct1 THEN
                        RecursiveSort(Low,Ct1);
                     IF Ct2 < High THEN
                        RecursiveSort(Ct2,High);
                END;
END;

(*****THE SORT ENDS HERE*****)

{get the files}

PROCEDURE GetFileList(    Template            : String;
                      VAR FilesFound          : Word);
          VAR
             Search : SearchRec;
          BEGIN
               FilesFound := 1;
               FindFirst(Template,32,Search);
               IF DosError <> 0 THEN BEGIN
                  NameArr^[1] := 'No Files';
               END
               ELSE BEGIN
                    NameArr^[1] := Search.Name;
                    REPEAT
                          FindNext(Search);
                          IF DosError = 0 THEN BEGIN
                             Inc(FilesFound);
                             NameArr^[FilesFound] := Search.Name;
                          END;
                    UNTIL (DosError <> 0) OR (FilesFound = MaxFileNames);
               END;
               RecursiveSort(1,FilesFound);
END;
PROCEDURE Transloc(     Loc                     : Word;
                    VAR X,Y                     : Byte);
                                                   { translates a numeric  }
      BEGIN
        X := ((Loc-1) DIV (HowDeep-1))*FileNameLen + UlCol;
        Y := (Loc-1) MOD (HowDeep-1) + UlRow;
      END;
PROCEDURE MoveScrPos(     Count                     : Word;
                      VAR X,Y                       : Byte;
                      VAR Loc                       : Word;
                      VAR Choice,EscChoice          : Char);
  VAR
     N,Ct             : Byte;
     Ctr              : Word;
   BEGIN
     Transloc(Loc,X,Y);
     GetKeys(Choice, EscChoice);
        IF Escchoice IN ['H','P','K','M'] THEN BEGIN
           CASE EscChoice OF
             'H': BEGIN
                    IF Loc > 1 THEN Loc := Loc - 1 ELSE Loc := Count;
                    Transloc(Loc,X,Y);
                  END;
                  { UP arrrow.  If the LOCation is any but the first, it
                    just decrements by one.  If it's the first, it becomes
                    the last.  Thus there's a "wrap" effect. }
             'P': BEGIN
                    IF Loc < Count THEN Loc := Loc + 1 ELSE Loc := 1;
                    Transloc(Loc,X,Y);
                  END;
                  { DOWN arrow.  Increments LOCation by one.  If already
                    at the end, "wraps" to beginning. }
             'K': BEGIN
                    IF Loc > (HowDeep - 1) THEN Loc := Loc - (HowDeep - 1) ELSE
                      BEGIN
                        Loc := Loc + 5*(HowDeep-1);
                        WHILE Loc > Count DO Loc := Loc - (HowDeep - 1);
                      END;
                    Transloc(Loc,X,Y);
                  END;
                  { LEFT arrow.  Moves to same screen line one column to
                    the left.  If already at leftmost column, goes to far
                    right column. }
             'M': BEGIN
                    Loc := Loc + HowDeep - 1;
                    IF Loc > Count THEN Loc := Loc MOD (HowDeep - 1) ;
                    IF Loc = 0 THEN Loc := HowDeep - 1;
                    Transloc(Loc,X,Y);
                  END;
                  { RIGHT arrow.  Moves to same screen line one column to
                    the right.  If already at rightmost, goes to far left. }
           END; {case}
         END;  { if }
END;
PROCEDURE WriteFileList(N,Start : Word;VAR Files : FileList;NameColor : Byte);
          VAR
             X,Y            : Byte;
             Ct             : Word;
          BEGIN
               IF N > MaxDisplay THEN
                  N := MaxDisplay;
               Dec(Start);
               FOR Ct := 1 TO n DO BEGIN
                   TransLoc(Ct,X,Y);
                   VarFWrite(GetScrPos(X,Y),NameColor,Files[Ct + Start]);
               END;
END;
PROCEDURE LightBar(    N,Start      : Word;
                   VAR Loc          : Word;
                   VAR Ch,Ch2       : Char;
                   VAR Files        : FileList;
                       NameColor,
                       BarColor     : Byte);
VAR
   x,y               : Byte;
   Posit,TempPosit   : Word;
BEGIN
     IF N > MaxDisplay THEN
        N := MaxDisplay;
     TransLoc(Loc,X,Y);
     Posit := GetScrPos(X,Y);
     SetAttr(Posit,Length(Files[Pred(Loc + Start)]),BarColor);
     REPEAT
          MoveScrPos(N,x,y,Loc,Ch,Ch2);
          TempPosit := GetScrPos(X,Y);
          IF Posit <> TempPosit THEN BEGIN
             SetAttr(Posit,12,NameColor);         {move light bar}
             SetAttr(TempPosit,Length(Files[Pred(Loc + Start)]),BarColor);
             Posit := TempPosit;
          END;
     UNTIL (Ch IN [#13,#27]) OR (Ch2 IN [#73,#81]);
END;
PROCEDURE SetFileListColors;
          BEGIN
               IF Color THEN BEGIN
                  BoxColor   := 112;
                  InnerColor := 63;
                  BarColor   := 28;
               END;
END;
FUNCTION  DrawFileList(    FileNo          : Word;
                       VAR Loc             : Word;       {light bar pos}
                       VAR FileOut         : String;
                       VAR Ch2             : Char;
                           ArrLoc          : Word;
                           Shadow          : Boolean):Char;
          VAR
             Ch  : Char;
             LRX : Byte;
          BEGIN
               SetCursorSize(NoCursor);
               SetFileListColors;
               FileNo := FileNo - Pred(ArrLoc);
               IF FileNo > MaxDisplay THEN
                  FileNo := MaxDisplay;
               LRX := UlCol + Succ(FileNo DIV Pred(HowDeep)) * FileNameLen;
               IF LRX > 79 THEN LRX := 79;
               MakeBox(ULCol-2,Pred(ULRow),LRX,
                       UlRow + Pred(HowDeep),BoxColor,InnerColor,BoxExt1);
               IF Shadow THEN
                  MakeShadow(ULCol-2,Pred(ULRow),LRX,UlRow + Pred(HowDeep));
               WriteFileList(FileNo,ArrLoc,NameArr^,InnerColor);
               LightBar(FileNo,ArrLoc,Loc,Ch,Ch2,NameArr^,InnerColor,BarColor);
               DrawFileList := Ch;
               FileOut := NameArr^[ArrLoc + Pred(Loc)];
               SetCursorSize(Cursor);
END;
PROCEDURE InitFileDisp(VAR Error : Boolean);
          BEGIN
               Error := FALSE;
               IF MaxAvail > LongInt(SizeOf(FileList)) + 10000 THEN
                  New(NameArr)
               ELSE
                   Error := TRUE;
END;
PROCEDURE DisposeFileDisp;
          BEGIN
               Dispose(NameArr);
END;

PROCEDURE PgUpDn(ArrNo,FileNo : Word;VAR Ptr : Pointer;Shadow : Boolean);
          VAR
             WindOn : Boolean;
             St : String[40];
          BEGIN
               Ptr := NIL;
               WindOn := FALSE;
               St := '';
               IF ArrNo + MaxDisplay < FileNo THEN BEGIN
                  St := 'PgDn for more, ';
                  WindOn := TRUE;
               END;
               IF ArrNo <> 0 THEN BEGIN
                  St := St + 'PgUp to go back ';
                  WindOn := TRUE;
               END;
               IF WindOn THEN
                  OpenOneLineWindow(Ptr,5,HowDeep + 4,Video1,Video2,
                                    BoxExt1,St,Shadow);
END;
PROCEDURE DoScroll(    Ch2           : Char;
                       ArrLoc,FileNo : Word;
                   VAR Scroll        : Boolean;
                   VAR Page,Loc      : Word);
          BEGIN
               Scroll := FALSE;
               CASE Ch2 OF
                        #73 :BEGIN
                                 IF Page <> 1 THEN BEGIN
                                    Dec(Page);
                                    Scroll := TRUE;
                                    Loc := 1;
                                 END;
                             END;
                        #81 :BEGIN
                                 IF (FileNo - ArrLoc) > MaxDisplay THEN BEGIN
                                     Inc(Page);
                                     Loc := 1;
                                     Scroll := TRUE;
                                 END;
                              END;
              END;
END;
PROCEDURE ScrollingFileList(    Template        : String;
                            VAR FileName        : String;
                            VAR Page,Loc        : Word;
                            VAR Result          : Byte;
                                Shadow          : Boolean);
          VAR
             WPtr1,WPtr2,WPtr3                 : Pointer;
             MemError,Scroll                   : Boolean;
             Ch,Ch2                            : Char;
             ArrNo,FileNo                      : Word;
          BEGIN
               Result := 3;               {3 = err, 2 = #27, 1 = OK}
               Scroll := FALSE;
               WPtr2 := NIL;
               FileName := '';
               InitFileDisp(MemError);
               IF MemError THEN BEGIN
                  WindowErrors(1);
                  Exit;
               END;
               GetFileList(Template,FileNo);
               IF (NameArr^[1] = 'No Files') OR (DOSError = 3) THEN BEGIN
                  WindowErrors(DOSError);
                  DisposeFileDisp;
                  Exit;
               END
               ELSE IF DOSError = 0 THEN            {not complete list}
                  WindowErrors(2);
               IF Shadow THEN
                   SaveWindow(1,1,80,HowDeep + 2,WPtr1)
               ELSE
                   SaveWindow(1,1,80,Succ(HowDeep),WPtr1);
               REPEAT

                   ArrNo := Pred(Page) * MaxDisplay;
                   IF (FileNo - ArrNo < MaxDisplay) AND Scroll THEN
                       ReturnBackground(WPtr1);
                   PgUpDn(ArrNo,FileNo,WPtr2,Shadow);
                   Ch := DrawFileList(FileNo,Loc,FileName,Ch2,Succ(ArrNo),Shadow);
                   RestoreWindow(WPtr2);
                   DoScroll(Ch2,ArrNo,FileNo,Scroll,Page,Loc);
                UNTIL Ch IN [#13,#27];
                RestoreWindow(WPtr1);
                DisposeFileDisp;
                IF Ch = #27 THEN
                   Result := 2
                ELSE
                   Result := 1;
END;
BEGIN
END.
