UNIT MCP;

INTERFACE

USES GlobType,Linlib,Utility;

PROCEDURE MakeMCP(    n      : Word;
                  VAR X,Y    : AkArr;
                  VAR NewN   : Word;
                  VAR Area   : Single);

IMPLEMENTATION

TYPE
    BoolArr = Array[1..2300] OF Boolean;

VAR
   TwoPi,
   PiDiv2,
   PiXThreeHalves : Single;

{Diddle checks the quadrant and adjusts the angle accordingly}

FUNCTION Diddle(x,y: Single):Single;
         BEGIN
              IF (X >= 0) AND (Y < 0) THEN          {quadrant 2}
                 Diddle := PiDiv2
              ELSE IF (Y >= 0) AND (X > 0) THEN
                   Diddle := Pi                     {quadrant 3}
              ELSE IF (x <= 0) AND (y > 0) THEN
                   Diddle := PiXThreeHalves         {quadrant 4}
              ELSE
                  Diddle := 0;                      {quadrant 1}
END;

{Get area is an assembly procedure that computes area based on the
 mapmaker's formula}

FUNCTION GetArea(Start,Stop : Word; VAR X,Y):Single; EXTERNAL;
{$L GetArea}

{MakeMCP produces a 100% MCP for any point set.  On return, X,Y hold
 the polygon coordinates.  NewN Holds the number of polygon coordinates
 and Area contains the area.  BE SURE TO SAVE X,Y somewhere if you
 need to use them after calling this procedure.}


PROCEDURE MakeMCP(    n      : Word;
                  VAR X,Y    : AkArr;
                  VAR NewN   : Word;
                  VAR Area   : Single);
          VAR
             Ct                : Word;
             Tx,Ty             : AkArr;    {21K stack requirements}
             Flag              : BoolArr;  {make sure that you have allocated}
             Yy,Xx,Direction,              {enough stack space}
             NewDir,DirDiff,
             MaxDir,TempDir,
             Holder            : Single;
             Start,i,j,Ii      : Word;
             XSw               : Boolean;
          LABEL
               GetNext;
          BEGIN
               ZeroFill(Flag,SizeOf(Flag));
               NewN := 0;
               Yy   := y[1];
               Start := 1;
               FOR Ct := 2 TO n DO BEGIN
                   IF Y[Ct] < Yy THEN BEGIN
                      Start := Ct;
                      Yy    := Y[Ct];    {find minimum y, an edge point}
                   END;
               END;
               Inc(NewN);
               Tx[1]     := X[Start];        {first pt in poly}
               Ty[1]     := Y[Start];
               i         := Start;
               Direction := Pi;              {point left <-}
               REPEAT
                     MaxDir := 1E30;             {a large number}
                     FOR Ct := 1 TO n DO BEGIN
                         IF (Flag[Ct] = FALSE) AND (Ct <> i) THEN BEGIN
                            Xx := X[i]-X[Ct];    {get the unit vectors}
                            Yy := Y[i]-Y[Ct];
                            XSw := (Xx = 0);
                            IF XSw AND (YY = 0) THEN
                                  GOTO GetNext;
                            IF XSw THEN
                               Xx := 0.0000001;
                            NewDir := ArcTan(YY/Xx);  {get the relative angle}
                            IF NewDir < 0 THEN        {if negative, adjust}
                               NewDir := PiDiv2 + NewDir;
                            NewDir := NewDir + Diddle(XX,YY);{adjust for quadrant}
                            DirDiff := Direction - NewDir;{choose angle closest}
                            IF DirDiff < 0 THEN           {to previous angle}
                               DirDiff := TwoPi + DirDiff;
                            IF (DirDiff < MaxDir)  THEN BEGIN
                               Ii := Ct;
                               MaxDir := DirDiff;  {save closest angle settings}
                               TempDir := NewDir;
                            END;
                        GetNext:
                         END;
                      END;
                      i := Ii;               {point found, reset for the}
                      Direction := TempDir;  {new point}
                      Flag[i] := TRUE;       {remove point from further}
                      Inc(NewN);             {consideration}
                      Tx[NewN] := X[i];
                      Ty[NewN] := Y[i];
               UNTIL i = Start;
               MoveRow(Tx,x,NewN);            {done, transfer from temp arr}
               MoveRow(Ty,y,NewN);            {linlib procedures}
               Area := 0;                            {compute area}
               Area := Area + GetArea(1,NewN,Tx,Ty); {assembly routine}
               Area := Area + (Tx[NewN]-Tx[1])*(Ty[NewN]+Ty[1]);{close it}
               Area := -Area * 0.5;         {oops, wrong way! take neg}
END;
BEGIN
     TwoPi := Pi*2;
     PiDiv2 := Pi * 0.5;
     PiXThreeHalves := Pi * 1.5;
END.







