UNIT WindLib;

INTERFACE

USES Crt,ScrLib,Utility,IOLib;

CONST
     MaxBarWindLen      = 5;
     BoxExt1            = 'ͻͼ';
     BoxExt2            = 'Ŀ';
TYPE
    String7        = String[7];
    WindStringArr  = Array[1..MaxBarWindLen] OF String[40];
    ColorArr       = Array[1..MaxBarWindLen] OF Byte;
    LightBarWindow =
                     RECORD
{holds saved screen}       WindArea    : Pointer;
{the actual entrys}        EntryNo     : Byte;
{words in window}          TextArr     : WindStringArr;
{Ok keystrokes}            LetterSt    : String[MaxBarWindLen];
{position of letter}       Position    : Array[1..MaxBarWindLen] OF Byte;
{border chars}             BoxBorder   : String7;
{color of border}          BoxColor    : Byte;
{color of text}            InnerColor  : Byte;
{highlighted text}         HiColor     : Byte;
{bar color}                BarColor    : Byte;
{highlighted bar color}    HiBarColor  : Byte;
{pos of the bar}           LightBarPos : Byte;
                     END;
FUNCTION InputInteger(    X,Y,Field : Byte;
                          Low,High  : Integer;
                      VAR Value     : Integer) : Boolean;
FUNCTION InputSingle(    X,Y,Field,Decimals   : Byte;
                          Low,High            : Integer;
                      VAR Value               : Single) : Boolean;
FUNCTION InputText(    X,Y,Field : Byte;
                   VAR St        : String) : Boolean;
PROCEDURE WindowErrors(Err : Byte);
PROCEDURE SetWindColors(VAR Window : LightBarWindow;VAR ColorSet : ColorArr);
FUNCTION WindLightBar(VAR Window : LightBarWindow) : Char;
PROCEDURE MakeBox(ULX,ULY,LRX,LRY,EdgeColor,InnerColor : Byte;
                  BoxString : String7);
PROCEDURE SaveWindow(ULX,ULY,LRX,LRY : Byte;VAR WindArea : Pointer);
PROCEDURE MakeShadow(X,Y,LRX,LRY : Word);
FUNCTION MakeWindow(VAR Ptr                      : Pointer;
                        ULX,ULY,LRX,
                        LRY,BoxColor,
                        InnerColor               : Byte;
                        Shadow                   : Boolean) : Boolean;
PROCEDURE RestoreWindow(VAR WindArea : Pointer);
PROCEDURE ReturnBackground(VAR WindArea : Pointer);
PROCEDURE OpenLtBarWindow(    ULX,ULY       : Byte;
                          VAR Window        : LightBarWindow;
                          VAR ColorSet      : ColorArr;
                              Shadow        : Boolean);
PROCEDURE OpenOneLineWindow(VAR Ptr                   : Pointer;
                                ULX,ULY               : Byte;
                                BoxColor,InnerColor   : Word;
                                BoxBorder             : String7;
                                St                    : String;
                                Shadow                : Boolean);
PROCEDURE OpenTextWindow(VAR Ptr                   : Pointer;
                             ULX,ULY,LRX,LRY       : Byte;
                             BoxColor,InnerColor   : Word;
                             BoxBorder             : String7;
                             Shadow                : Boolean);
PROCEDURE Wait(VAR Ptr : Pointer);

IMPLEMENTATION
PROCEDURE WindowErrors(Err : Byte);
          VAR
              St  : String;
              Ptr : Pointer;
          BEGIN
               CASE Err OF
                    1 : St := 'Error - Not enough memory';
                    2 : St := 'Note - Incomplete file List';
                    3 : St := 'Error - Directory does not exist';
                    4 : St := 'Error - Path not found';
                    5 : St := 'No file selected';
                   18 : St := 'File not found';
                   100 : St := 'Too many points';
                   101 : St := 'Too few points';
                   102 : St := 'Problem with file write';
                   ELSE BEGIN
                       Str(Err,St);
                       St := 'DOS Error # ' + St;
                   END;
               END;
               OpenOneLineWindow(Ptr,(77 - Byte(St[0])) SHR 1,9,ErrorVid,
                                 ErrorVid,BoxExt1,St,TRUE); {no shadow}
               Delay(1000);
               RestoreWindow(Ptr);
END;
FUNCTION DimError(x,y : Byte):Boolean;
         VAR
            Temp : Boolean;
         BEGIN
              Temp := ((x > 80) OR (y > 25));
              IF Temp THEN
                 WriteLn('Error - window dimensions off screen');
              DimError := Temp;
END;

{GetMaxLen Returns the maximum string length in a window record}

FUNCTION GetMaxLen(Entries : Byte;VAR Arr : WindStringArr) : Byte;
         VAR
            Ct,Len : Byte;
         BEGIN
              Len := 0;
              FOR Ct := 1 TO Entries DO
                  IF Byte(Arr[Ct,0]) > Len THEN
                     Len := Byte(Arr[Ct,0]);
              GetMaxLen := Len;
END;

{SetWindowColors sets alternate colors to a light bar window if Color = True}

PROCEDURE SetWindColors(VAR Window : LightBarWindow;VAR ColorSet : ColorArr);
          BEGIN
               IF Color THEN BEGIN
                  WITH Window DO BEGIN
                       BoxColor    := ColorSet[1];
                       InnerColor  := ColorSet[2];
                       HiColor     := ColorSet[3];
                       BarColor    := ColorSet[4];
                       HiBarColor  := ColorSet[5];
                  END;
               END;
END;

{Writes a string with a single character in an alternate color}

PROCEDURE HiWrite(    Posit                            : Word;
                      CharPosit,Color,HiColor          : Byte;
                  VAR St                               : String);
          BEGIN
               VarFWrite(Posit,Color,St);
               WriteChar(Posit + (Pred(CharPosit) * 2),St[CharPosit],HiColor);
END;

{MakeBox draws a filled box on the screen}

PROCEDURE MakeBox(ULX,ULY,LRX,LRY,EdgeColor,InnerColor : Byte;
                  BoxString : String7);
          VAR
             Posit1,Posit2,Len,Width,
             WidthM2                       : Word;
             Ct                            : Byte;
          BEGIN
               Width := (Succ(LRX) - ULX);
               Len   := Succ(LRY) - ULY;           {window size}
               WidthM2 := Width - 2;
               Width := Pred(Width) SHL 1;
               Posit1 := GetScrPos(ULX,ULY);
               Posit2 := Posit1 + Width;
               WriteChar(Posit1,BoxString[1],EdgeColor);
               CharToScreen(Posit1 + 2,WidthM2,BoxString[2],EdgeColor);
               WriteChar(Posit2,BoxString[3],EdgeColor);
               FOR Ct := 1 TO Len - 2 DO BEGIN
                   Inc(Posit1,160);
                   Inc(Posit2,160);
                   WriteChar(Posit1,BoxString[4],EdgeColor);
                   CharToScreen(Posit1 + 2,WidthM2,' ',InnerColor);
                   WriteChar(Posit2,BoxString[4],EdgeColor);
               END;
               Inc(Posit1,160);
               Inc(Posit2,160);
               WriteChar(Posit1,BoxString[5],EdgeColor);
               CharToScreen(Posit1 + 2,WidthM2,BoxString[6],EdgeColor);
               WriteChar(Posit2,BoxString[7],EdgeColor);
END;

{Note : these two procs provide sophisticated memory management for
 windows. Save window determines the size of the window and assigns
 exactly enough room on the heap to hold the window, its position and
 its dimensions.  Restore window retrives the window record, restores
 the screen area and disposes of the ptr.}

PROCEDURE SaveWindow(ULX,ULY,LRX,LRY : Byte;VAR WindArea : Pointer);
          VAR
             Ct                : Byte;
             Posit,Width,Len,
             WidthX2           : Word;
             TempPtr           : Pointer;
             PtrOfs            : Word ABSOLUTE TempPtr;
             TempArr           : Array[1..3] OF Word;
          BEGIN
               Posit := GetScrPos(ULX,ULY);
               Width := (Succ(LRX) - ULX);
               Len   := Succ(LRY) - ULY;               {window size}
               WidthX2 := Width * 2;
               GetMem(WindArea,6 + Len * WidthX2);   {assign only enough space}
               TempPtr := WindArea;
               TempArr[1] := Len;                  {save len,width and posit}
               TempArr[2] := Width;
               TempArr[3] := Posit;
               FastMove(TempArr,TempPtr^,6);       {move them to ptr^}
               Inc(PtrOfs,6);                      {inc ptr address,6}
               FOR Ct := 1 TO Len DO BEGIN
                   BlockFromScreen(TempPtr^,Posit,Width);  {move each row}
                   Inc(Posit,160);                         {inc screen loc}
                   Inc(PtrOfs,WidthX2);                    {inc ptr}
               END;
END;

{Restores the screen saved i  OpenWindow and disposes of the heap space}

PROCEDURE RestoreWindow(VAR WindArea : Pointer);
          VAR
             Ct                : Byte;
             TempPtr           : Pointer;
             PtrOfs            : Word ABSOLUTE TempPtr;
             TempArr           : Array[1..3] OF Word;
             WidthX2           : Word;
          BEGIN
               IF WindArea = NIL THEN Exit;     {if window is not assigned}
               TempPtr := WindArea;             {then exit}
               FastMove(TempPtr^,TempArr,6);    {get size data}
               Inc(PtrOfs,6);                   {inc ptr^ past size data}
               WidthX2 := TempArr[2] * 2;
               FOR Ct := 1 TO TempArr[1] DO BEGIN
                   BlockToScreen(TempPtr^,TempArr[3],TempArr[2]);{screen back}
                   Inc(TempArr[3],160);                    {inc screen pos}
                   Inc(PtrOfs,WidthX2);                    {inc ptr^}
               END;
               FreeMem(WindArea,6 + TempArr[1] * WidthX2); {dispose ptr}
END;
PROCEDURE ReturnBackground(VAR WindArea : Pointer);
          VAR
             Ct                : Byte;
             TempPtr           : Pointer;
             PtrOfs            : Word ABSOLUTE TempPtr;
             TempArr           : Array[1..3] OF Word;
             WidthX2           : Word;
          BEGIN
               IF WindArea = NIL THEN Exit;     {if window is not assigned}
               TempPtr := WindArea;             {then exit}
               FastMove(TempPtr^,TempArr,6);    {get size data}
               Inc(PtrOfs,6);                   {inc ptr^ past size data}
               WidthX2 := TempArr[2] * 2;
               FOR Ct := 1 TO TempArr[1] DO BEGIN
                   BlockToScreen(TempPtr^,TempArr[3],TempArr[2]);{screen back}
                   Inc(TempArr[3],160);                    {inc screen pos}
                   Inc(PtrOfs,WidthX2);                    {inc ptr^}
               END;
END;

{called by makeshadow, makes a shadow color}

FUNCTION NewColor(Posit: Word):Byte;
         VAR
            Color : Byte;
         BEGIN
              Color := GetAttr(Posit) MOD 16; {black background}
              IF Color > 8 THEN
                 Dec(Color,8);                {low intensity}
              NewColor := Color;
END;

{makes a 'shadow' on the bottom, right sides of a box by shifting the
 attribute codes}

PROCEDURE MakeShadow(X,Y,LRX,LRY : Word);
          VAR
             Ct    : Byte;
             Posit : Word;
          BEGIN
               Posit := GetScrPos(LRX+1,Y+1);
               FOR Ct := Y TO LRY DO BEGIN
                   SetAttr(Posit,1,NewColor(Posit));
                   Inc(Posit,160);
               END;
               Posit := GetScrPos(X+1,LRY+1);
               FOR Ct := X TO LRX DO BEGIN
                   SetAttr(Posit,1,NewColor(Posit));
                   Inc(Posit,2);
               END;
END;

{saves a piece of the screen to Ptr, draws a box and a shadow}

FUNCTION MakeShadowedWindow(VAR Ptr                      : Pointer;
                                ULX,ULY,LRX,
                                LRY,BoxOutline,
                                BoxInterior              : Byte) : Boolean;
          VAR
             Error : Boolean;
          BEGIN
               Error := DimError(LRX+1,LRY+1);
               IF Error THEN Exit;
               SaveWindow(ULX,ULY,LRX+1,LRY+1,Ptr);
               MakeBox(ULX,ULY,LRX,LRY,BoxOutline,BoxInterior,BoxExt1);
               MakeShadow(ULX,ULY,LRX,LRY);
               MakeShadowedWindow := Error;
END;
FUNCTION MakeNormWindow(VAR Ptr                      : Pointer;
                            ULX,ULY,LRX,
                            LRY,BoxOutline,
                            BoxInterior              : Byte) : Boolean;
          VAR
             Error : Boolean;
          BEGIN
               Error := DimError(LRX,LRY);
               IF Error THEN Exit;
               SaveWindow(ULX,ULY,LRX,LRY,Ptr);
               MakeBox(ULX,ULY,LRX,LRY,BoxOutline,BoxInterior,BoxExt1);
               MakeNormWindow := Error;
END;
FUNCTION MakeWindow(VAR Ptr                      : Pointer;
                        ULX,ULY,LRX,
                        LRY,BoxColor,
                        InnerColor               : Byte;
                        Shadow                   : Boolean) : Boolean;
         VAR
            Error : Boolean;
         BEGIN
              IF Shadow THEN
                 Error :=  MakeShadowedWindow(Ptr,ULX,ULY,LRX,LRY,
                                              BoxColor,InnerColor)
              ELSE
                  Error := MakeNormWindow(Ptr,ULX,ULY,LRX,LRY,
                                          BoxColor,InnerColor);
              MakeWindow := Error;
END;
PROCEDURE OpenLtBarWindow(ULX,ULY       : Byte;
                          VAR Window    : LightBarWindow;
                          VAR ColorSet  : ColorArr;
                              Shadow    : Boolean);
          VAR
             LRX,LRY    : Byte;
             Posit,Ct   : Word;
          BEGIN
               SetWindColors(Window,ColorSet);
               WITH Window DO BEGIN
                    LRX := ULX + GetMaxLen(EntryNo,TextArr) + 3;
                    LRY := ULY + EntryNo + 1;
                    IF MakeWindow(WindArea,ULX,ULY,LRX,LRY,
                                  BoxColor,InnerColor,Shadow)
                      THEN Exit;
                    Posit := GetScrPos(ULX + 2,ULY + 1);
                    FOR Ct := 1 TO EntryNo DO BEGIN
                        HiWrite(Posit,Position[Ct],InnerColor,HiColor,
                                TextArr[Ct]);
                        Inc(Posit,160);
                    END;
               END;
END;

{performs light bar operation and returns a character - LightBarPos is
 altered by this procedure}

FUNCTION WindLightBar(VAR Window : LightBarWindow) : Char;
          VAR
             TopPosit,
             Posit    : Word;
             LetPos   : Byte;
             TempPtr  : Pointer;
             TempOfs  : Word ABSOLUTE TempPtr;
             Ch,Ch2   : Char;
          LABEL
               Finis;
          BEGIN
               WITH Window DO BEGIN
                    TempPtr := WindArea;
                    Inc(TempOfs,4);
                    FastMove(TempPtr^,TopPosit,2);    {get the start pos}
                    Inc(TopPosit,164);                {move to top text}
                    Posit := TopPosit + (160 * Pred(LightBarPos));
{write first bar}   HiWrite(Posit,Position[LightBarPos],BarColor,HiBarColor,
                            TextArr[LightBarPos]);
                 REPEAT    {endless loop}
                    REPEAT
                          GetKeys(Ch,Ch2);
                          Ch := UpCase(Ch);
                          LetPos := CharPos(Ch,LetterSt); {ok letter}
                    UNTIL (LetPos <> 0) OR (Ch IN [#27,#13])
                          OR (Ch2 IN [#72,#80]);
{erase bar}         HiWrite(Posit,Position[LightBarPos],InnerColor,HiColor,
                            TextArr[LightBarPos]);
                    CASE Ch OF
                         #27   : BEGIN
                                       WindLightBar := Ch;
                                       Exit;
                                  END;
                          #13   : BEGIN
                                       WindLightBar := LetterSt[LightBarPos];
                                       Exit;
                                  END;
                         #0     : BEGIN              {Up,Dn}
                                       CASE Ch2 OF
                                            #72  : BEGIN    {Up}
                                                       Dec(LightBarPos);
                                                       IF LightBarPos = 0
                                                         THEN
                                                         LightBarPos := EntryNo;
                                                   END;
                                            #80  : BEGIN    {Dn}
                                                       Inc(LightBarPos);
                                                       IF LightBarPos >
                                                         EntryNo THEN
                                                         LightBarPos := 1;
                                                   END;
                                       END;
                                  END;
                         ELSE BEGIN
                                   LightBarPos := LetPos;
                                   WindLightBar := LetterSt[LetPos];
                                   Exit;
                              END;
                    END;
                    Posit := TopPosit + (160 * Pred(LightBarPos));
{write new bar}     HiWrite(Posit,Position[LightBarPos],BarColor,HiBarColor,
                            TextArr[LightBarPos]);
                 UNTIL FALSE;
              END;
END;

{Opens a window and writes a single line of text to it - self scaling}

PROCEDURE OpenOneLineWindow(VAR Ptr                   : Pointer;
                                ULX,ULY               : Byte;
                                BoxColor,InnerColor   : Word;
                                BoxBorder             : String7;
                                St                    : String;
                                Shadow                : Boolean);
          VAR
             LRX,LRY : Byte;
          BEGIN
               Ptr := NIL;
               LRX := ULX + Byte(St[0]) + 3;
               LRY := ULY + 2;
               IF MakeWindow(Ptr,ULX,ULY,LRX,LRY,BoxColor,InnerColor,Shadow)
                  THEN Exit;
               VarFWrite(GetScrPos(ULX + 2,ULY + 1),InnerColor,St);
END;

{Opens a text window and declares a turbo pascal window within it -
 you must remember to set the window out to (1,1,80,25) at any point at
 which you wish to use Write to write to a portion of the screen exterior
 to the window. - Use this window for dialog.}

PROCEDURE OpenTextWindow(VAR Ptr                   : Pointer;
                             ULX,ULY,LRX,LRY       : Byte;
                             BoxColor,InnerColor   : Word;
                             BoxBorder             : String7;
                             Shadow                : Boolean);
          BEGIN
               Ptr := NIL;
               IF MakeWindow(Ptr,ULX,ULY,LRX,LRY,BoxColor,InnerColor,Shadow)
                  THEN Exit;
               Window(ULX+1,ULY+1,LRX-1,LRY-1);
END;
FUNCTION InputErrors(Low,High : Single;ErrorNo : Byte) : Boolean;
          VAR
             Ch,Ch2      : Char;
             Temp1,Temp2 : String[20];
             Ptr         : Pointer;
          BEGIN
               SetCursorSize(NoCursor);
               SaveWindow(20,8,61,12,Ptr);
               MakeBox(20,8,61,12,ErrorVid,ErrorVid,BoxExt2);
               CASE ErrorNo OF
                   1 : CenterText(9,ErrorVid,'Error in Numeric Format');
                   2 : BEGIN
                           Str(Low:2:2,Temp1);
                           Str(High:2:2,Temp2);
                           CenterText(9,ErrorVid,'Error - Value out of Bounds');
                           CenterText(10,ErrorVid,Temp1 + '  X  '+ Temp2);
                        END;
               END;
               CenterText(11,ErrorVid,'Hit Any Key to Continue, <ESC> to Quit');
               GetKeys(Ch,Ch2);
               InputErrors := (Ch = #27);  {an error returned}
               RestoreWindow(Ptr);
               SetCursorSize(Cursor);
END;
FUNCTION InputInteger(     X,Y,Field : Byte;
                           Low,High  : Integer;
                       VAR Value     : Integer) : Boolean;
         VAR
              St     : String[20];
              Ch,Ch2 : Char;
              Err    : Integer;
              Holder : Integer;
         LABEL
              LOOP;
         BEGIN
              Holder := Value;
             LOOP:
              St := '                    ';
              GoToXY(X,Y);
              St[0] := Char(Field);
              Write(St);
              GoToXY(X,Y);
              St[0] := #0;
              REPEAT
                    GetKeys(Ch,Ch2);
                    IF (Ch IN ['0'..'9','-']) AND
                       (St[0] < Char(Field)) THEN BEGIN
                       St := St + Ch;
                       Write(Ch);
                    END;
                    IF (Ch = #8) AND (St[0] <> #0) THEN BEGIN
                          Dec(St[0]);
                          Write(Ch);
                          Write(#32);
                          Write(Ch);
                    END;
                    IF Ch = #27 THEN BEGIN
                       InputInteger := TRUE;
                       Exit;
                    END;
              UNTIL Ch = #13;
              IF St[0] = #0 THEN BEGIN
                  GoToXY(X,Y);
                  Write(Value:Field);
                  InputInteger := FALSE;
              END
              ELSE BEGIN
                   Val(St,Holder,Err);
                   IF Err <> 0 THEN BEGIN
                      IF InputErrors(Low,High,1) THEN BEGIN
                         InputInteger := TRUE;
                         Exit;
                      END;
                      GOTO LOOP;
                   END;
                   IF (Holder < Low) OR (Holder > High) THEN BEGIN
                      IF InputErrors(Low,High,2) THEN BEGIN
                         InputInteger := TRUE;
                         Exit;
                      END;
                      GOTO LOOP;
                   END;
                   Value        := Holder;
                   InputInteger := FALSE;
             END;
END;
FUNCTION InputSingle(     X,Y,Field,Decimals  : Byte;
                           Low,High           : Integer;
                       VAR Value              : Single) : Boolean;
         VAR
              St     : String[20];
              Ch,Ch2 : Char;
              Err    : Integer;
              Holder : Single;
         LABEL
              LOOP;
         BEGIN
              Holder := Value;
             LOOP:
              St := '                    ';
              GoToXY(X,Y);
              St[0] := Char(Field);
              Write(St);
              GoToXY(X,Y);
              St[0] := #0;
              REPEAT
                    GetKeys(Ch,Ch2);
                    IF (Ch IN ['0'..'9','-','.']) AND
                       (St[0] < Char(Field)) THEN BEGIN
                       St := St + Ch;
                       Write(Ch);
                    END;
                    IF (Ch = #8) AND (St[0] <> #0) THEN BEGIN
                          Dec(St[0]);
                          Write(Ch);
                          Write(#32);
                          Write(Ch);
                    END;
                    IF Ch = #27 THEN BEGIN
                       InputSingle := TRUE;
                       Exit;
                    END;
              UNTIL Ch = #13;
              IF St[0] = #0 THEN BEGIN
                  GoToXY(X,Y);
                  Write(Value:Field:Decimals);
                  InputSingle := FALSE;
              END
              ELSE BEGIN
                   IF St[1] = '.' THEN
                      St := '0' + St;
                   Val(St,Holder,Err);
                   IF Err <> 0 THEN BEGIN
                      IF InputErrors(Low,High,1) THEN BEGIN
                         InputSingle := TRUE;
                         Exit;
                      END;
                      GOTO LOOP;
                   END;
                   IF (Holder < Low) OR (Holder > High) THEN BEGIN
                      IF InputErrors(Low,High,2) THEN BEGIN
                         InputSingle := TRUE;
                         Exit;
                      END;
                      GOTO LOOP;
                   END;
                   Value        := Holder;
                   InputSingle := FALSE;
             END;
END;
FUNCTION InputText(     X,Y,Field : Byte;
                    VAR St        : String) : Boolean;
         VAR
              Ch,Ch2 : Char;
         LABEL
              LOOP;
         BEGIN
             LOOP:
              GoToXY(X,Y);
              St[0] := #0;
              REPEAT
                    GetKeys(Ch,Ch2);
                    IF NOT (Ch IN [#0,#8,#27,#13]) AND (Byte(St[0]) <> Field)
                       THEN BEGIN
                       Inc(St[0]);
                       St[Byte(St[0])] :=  Ch;
                       Write(Ch);
                    END;
                    IF (Ch = #8) AND (St[0] <> #0) THEN BEGIN
                          Dec(St[0]);
                          Write(Ch);
                          Write(#32);
                          Write(Ch);
                          IF WhereX = 1 THEN BEGIN
                             GoToXY(X,Y);
                             Write(St);
                             WriteChar(GetScrPos(WhereX + Lo(WindMin),
                                                 WhereY + Hi(WindMin)),#32,
                                                 TextAttr);
                          END;
                    END;
                    IF Ch = #27 THEN BEGIN
                       InputText := TRUE;
                       Exit;
                    END;
              UNTIL Ch = #13;
              InputText := FALSE;
END;
PROCEDURE Wait(VAR Ptr : Pointer);
          BEGIN
               OpenOneLineWindow(Ptr,70,3,ErrorVid,ErrorVid+Blink,
                                 BoxExt2,'WAIT',TRUE)
END;

BEGIN
END.